==> jei-1.21.10-fabric-26.1.1.25-sources.jar: mezz/jei/library/util/BrewingRecipeMakerCommon.java <==
package mezz.jei.library.util;

import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.ingredients.IIngredientHelper;
import mezz.jei.api.ingredients.subtypes.UidContext;
import mezz.jei.api.recipe.vanilla.IJeiBrewingRecipe;
import mezz.jei.api.recipe.vanilla.IVanillaRecipeFactory;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.platform.IPlatformIngredientHelper;
import mezz.jei.common.platform.Services;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.RegistryUtil;
import mezz.jei.library.ingredients.IngredientSet;
import mezz.jei.library.plugins.vanilla.ingredients.subtypes.PotionSubtypeInterpreter;
import net.minecraft.class_10302;
import net.minecraft.class_10352;
import net.minecraft.class_10363;
import net.minecraft.class_1799;
import net.minecraft.class_1812;
import net.minecraft.class_1842;
import net.minecraft.class_1844;
import net.minecraft.class_1845;
import net.minecraft.class_1856;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_6880;
import net.minecraft.class_7924;
import net.minecraft.class_9334;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

public class BrewingRecipeMakerCommon {
	private static final Logger LOGGER = LogManager.getLogger();

	public static Set<IJeiBrewingRecipe> getVanillaBrewingRecipes(
		IVanillaRecipeFactory recipeFactory,
		IIngredientManager ingredientManager,
		class_1845 potionBrewing
	) {
		Set<IJeiBrewingRecipe> recipes = new HashSet<>();
		class_2378<class_1842> potionRegistry = RegistryUtil.getRegistry(class_7924.field_41215);
		IIngredientHelper<class_1799> itemStackHelper = ingredientManager.getIngredientHelper(VanillaTypes.ITEM_STACK);

		class_310 minecraft = class_310.method_1551();
		class_10352 contextmap = class_10363.method_65008(Objects.requireNonNull(minecraft.field_1687));

		IngredientSet<class_1799> knownPotions = getBaseKnownPotions(ingredientManager, potionRegistry, potionBrewing, contextmap);

		IPlatformIngredientHelper ingredientHelper = Services.PLATFORM.getIngredientHelper();
		IngredientSet<class_1799> potionReagents = ingredientHelper.getPotionIngredients(potionBrewing)
			.map(class_1856::method_64673)
			.flatMap(display -> display.method_64739(contextmap, class_10302.class_10361.field_54901))
			.collect(Collectors.toCollection(() -> new IngredientSet<>(itemStackHelper, UidContext.Ingredient)));

		boolean foundNewPotions;
		do {
			List<class_1799> newPotions = getNewPotions(
				potionBrewing,
				recipeFactory,
				itemStackHelper,
				knownPotions,
				potionReagents,
				recipes
			);
			foundNewPotions = !newPotions.isEmpty();
			knownPotions.addAll(newPotions);
		} while (foundNewPotions);

		return recipes;
	}

	private static IngredientSet<class_1799> getBaseKnownPotions(
		IIngredientManager ingredientManager,
		class_2378<class_1842> potionRegistry,
		class_1845 potionBrewing,
		class_10352 contextmap
	) {
		IPlatformIngredientHelper ingredientHelper = Services.PLATFORM.getIngredientHelper();
		IIngredientHelper<class_1799> itemStackHelper = ingredientManager.getIngredientHelper(VanillaTypes.ITEM_STACK);

		IngredientSet<class_1799> potionContainers = ingredientHelper
			.getPotionContainers(potionBrewing)
			.stream()
			.map(class_1856::method_64673)
			.flatMap(display -> display.method_64739(contextmap, class_10302.class_10361.field_54901))
			.collect(Collectors.toCollection(() -> new IngredientSet<>(itemStackHelper, UidContext.Ingredient)));

		IngredientSet<class_1799> knownPotions = new IngredientSet<>(itemStackHelper, UidContext.Ingredient);
		knownPotions.addAll(potionContainers);

		potionRegistry.method_42017()
			.forEach(potion -> {
				for (class_1799 potionContainer : potionContainers) {
					class_1799 result = class_1844.method_57400(potionContainer.method_7909(), potion);
					knownPotions.add(result);
				}
			});
		return knownPotions;
	}

	private static List<class_1799> getNewPotions(
		class_1845 potionBrewing,
		IVanillaRecipeFactory recipeFactory,
		IIngredientHelper<class_1799> itemStackHelper,
		Collection<class_1799> knownPotions,
		Collection<class_1799> potionReagents,
		Collection<IJeiBrewingRecipe> recipes
	) {
		List<class_1799> newPotions = new ArrayList<>();
		for (class_1799 potionInput : knownPotions) {
			Object inputId = itemStackHelper.getUid(potionInput, UidContext.Recipe);
			String inputPathId = PotionSubtypeInterpreter.INSTANCE.getStringName(potionInput);

			for (class_1799 potionReagent : potionReagents) {
				class_1799 potionInputCopy = potionInput.method_7972();
				class_1799 potionOutput = getOutput(potionBrewing, potionInputCopy, potionReagent);
				if (potionOutput.method_7960()) {
					continue;
				}

				if (potionInput.method_7909() instanceof class_1812 && potionOutput.method_7909() instanceof class_1812) {
					Optional<class_6880<class_1842>> potionOutputType = potionOutput.method_58695(class_9334.field_49651, class_1844.field_49274).comp_2378();
					if (potionOutputType.isEmpty()) {
						continue;
					}
				}

				Object outputId = itemStackHelper.getUid(potionOutput, UidContext.Recipe);
				if (Objects.equals(inputId, outputId)) {
					continue;
				}

				class_2960 outputResourceLocation = itemStackHelper.getResourceLocation(potionOutput);
				String outputPathId = PotionSubtypeInterpreter.INSTANCE.getStringName(potionOutput);
				String outputModId = outputResourceLocation.method_12836();
				String uidPath = ResourceLocationUtil.sanitizePath(inputPathId + ".to." + outputPathId);
				IJeiBrewingRecipe recipe = recipeFactory.createBrewingRecipe(
					List.of(potionReagent),
					potionInputCopy,
					potionOutput,
					class_2960.method_60655(outputModId, uidPath)
				);

				IJeiBrewingRecipe existingRecipe = recipes.stream()
					.filter(recipe::equals)
					.findFirst()
					.orElse(null);
				if (existingRecipe == null) {
					recipes.add(recipe);
					newPotions.add(potionOutput);
				} else {
					// This is a recipe with the same uid and output as an existing recipe,
					// but it has a different reagent.
					// Create a recipe that combines the two.
					IngredientSet<class_1799> reagents = new IngredientSet<>(itemStackHelper, UidContext.Recipe);
					reagents.addAll(existingRecipe.getIngredients());
					reagents.add(potionReagent);
					if (reagents.size() != existingRecipe.getIngredients().size()) {
						IJeiBrewingRecipe replacementRecipe = recipeFactory.createBrewingRecipe(
							List.copyOf(reagents),
							existingRecipe.getPotionInputs(),
							existingRecipe.getPotionOutput(),
							existingRecipe.getUid()
						);
						recipes.remove(existingRecipe);
						recipes.add(replacementRecipe);
					}
				}
			}
		}
		return newPotions;
	}

	private static class_1799 getOutput(class_1845 potionBrewing, class_1799 potion, class_1799 itemStack) {
		try {
			class_1799 result = potionBrewing.method_8078(itemStack, potion);
			if (result != itemStack) {
				return result;
			}
		} catch (RuntimeException e) {
			String potionInfo = ErrorUtil.getItemStackInfo(potion);
			String itemStackInfo = ErrorUtil.getItemStackInfo(itemStack);
			LOGGER.error(
				"A modded potion mix crashed: \nPotion: {}\nItemStack: {}",
				potionInfo,
				itemStackInfo,
				e
			);
		}
		return class_1799.field_8037;
	}
}

==> jei-1.21.11-fabric-27.0.0.1-sources.jar: mezz/jei/library/util/BrewingRecipeMakerCommon.java <==
package mezz.jei.library.util;

import mezz.jei.api.constants.VanillaTypes;
import mezz.jei.api.ingredients.IIngredientHelper;
import mezz.jei.api.ingredients.subtypes.UidContext;
import mezz.jei.api.recipe.vanilla.IJeiBrewingRecipe;
import mezz.jei.api.recipe.vanilla.IVanillaRecipeFactory;
import mezz.jei.api.runtime.IIngredientManager;
import mezz.jei.common.platform.IPlatformIngredientHelper;
import mezz.jei.common.platform.Services;
import mezz.jei.common.util.ErrorUtil;
import mezz.jei.common.util.RegistryUtil;
import mezz.jei.library.ingredients.IngredientSet;
import mezz.jei.library.plugins.vanilla.ingredients.subtypes.PotionSubtypeInterpreter;
import net.minecraft.class_10302;
import net.minecraft.class_10352;
import net.minecraft.class_10363;
import net.minecraft.class_1799;
import net.minecraft.class_1812;
import net.minecraft.class_1842;
import net.minecraft.class_1844;
import net.minecraft.class_1845;
import net.minecraft.class_1856;
import net.minecraft.class_2378;
import net.minecraft.class_2960;
import net.minecraft.class_310;
import net.minecraft.class_6880;
import net.minecraft.class_7924;
import net.minecraft.class_9334;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Set;
import java.util.stream.Collectors;

public class BrewingRecipeMakerCommon {
	private static final Logger LOGGER = LogManager.getLogger();

	public static Set<IJeiBrewingRecipe> getVanillaBrewingRecipes(
		IVanillaRecipeFactory recipeFactory,
		IIngredientManager ingredientManager,
		class_1845 potionBrewing
	) {
		Set<IJeiBrewingRecipe> recipes = new HashSet<>();
		class_2378<class_1842> potionRegistry = RegistryUtil.getRegistry(class_7924.field_41215);
		IIngredientHelper<class_1799> itemStackHelper = ingredientManager.getIngredientHelper(VanillaTypes.ITEM_STACK);

		class_310 minecraft = class_310.method_1551();
		class_10352 contextmap = class_10363.method_65008(Objects.requireNonNull(minecraft.field_1687));

		IngredientSet<class_1799> knownPotions = getBaseKnownPotions(ingredientManager, potionRegistry, potionBrewing, contextmap);

		IPlatformIngredientHelper ingredientHelper = Services.PLATFORM.getIngredientHelper();
		IngredientSet<class_1799> potionReagents = ingredientHelper.getPotionIngredients(potionBrewing)
			.map(class_1856::method_64673)
			.flatMap(display -> display.method_64739(contextmap, class_10302.class_10361.field_54901))
			.collect(Collectors.toCollection(() -> new IngredientSet<>(itemStackHelper, UidContext.Ingredient)));

		boolean foundNewPotions;
		do {
			List<class_1799> newPotions = getNewPotions(
				potionBrewing,
				recipeFactory,
				itemStackHelper,
				knownPotions,
				potionReagents,
				recipes
			);
			foundNewPotions = !newPotions.isEmpty();
			knownPotions.addAll(newPotions);
		} while (foundNewPotions);

		return recipes;
	}

	private static IngredientSet<class_1799> getBaseKnownPotions(
		IIngredientManager ingredientManager,
		class_2378<class_1842> potionRegistry,
		class_1845 potionBrewing,
		class_10352 contextmap
	) {
		IPlatformIngredientHelper ingredientHelper = Services.PLATFORM.getIngredientHelper();
		IIngredientHelper<class_1799> itemStackHelper = ingredientManager.getIngredientHelper(VanillaTypes.ITEM_STACK);

		IngredientSet<class_1799> potionContainers = ingredientHelper
			.getPotionContainers(potionBrewing)
			.stream()
			.map(class_1856::method_64673)
			.flatMap(display -> display.method_64739(contextmap, class_10302.class_10361.field_54901))
			.collect(Collectors.toCollection(() -> new IngredientSet<>(itemStackHelper, UidContext.Ingredient)));

		IngredientSet<class_1799> knownPotions = new IngredientSet<>(itemStackHelper, UidContext.Ingredient);
		knownPotions.addAll(potionContainers);

		potionRegistry.method_42017()
			.forEach(potion -> {
				for (class_1799 potionContainer : potionContainers) {
					class_1799 result = class_1844.method_57400(potionContainer.method_7909(), potion);
					knownPotions.add(result);
				}
			});
		return knownPotions;
	}

	private static List<class_1799> getNewPotions(
		class_1845 potionBrewing,
		IVanillaRecipeFactory recipeFactory,
		IIngredientHelper<class_1799> itemStackHelper,
		Collection<class_1799> knownPotions,
		Collection<class_1799> potionReagents,
		Collection<IJeiBrewingRecipe> recipes
	) {
		List<class_1799> newPotions = new ArrayList<>();
		for (class_1799 potionInput : knownPotions) {
			Object inputId = itemStackHelper.getUid(potionInput, UidContext.Recipe);
			String inputPathId = PotionSubtypeInterpreter.INSTANCE.getStringName(potionInput);

			for (class_1799 potionReagent : potionReagents) {
				class_1799 potionInputCopy = potionInput.method_7972();
				class_1799 potionOutput = getOutput(potionBrewing, potionInputCopy, potionReagent);
				if (potionOutput.method_7960()) {
					continue;
				}

				if (potionInput.method_7909() instanceof class_1812 && potionOutput.method_7909() instanceof class_1812) {
					Optional<class_6880<class_1842>> potionOutputType = potionOutput.method_58695(class_9334.field_49651, class_1844.field_49274).comp_2378();
					if (potionOutputType.isEmpty()) {
						continue;
					}
				}

				Object outputId = itemStackHelper.getUid(potionOutput, UidContext.Recipe);
				if (Objects.equals(inputId, outputId)) {
					continue;
				}

				class_2960 outputResourceLocation = itemStackHelper.getIdentifier(potionOutput);
				String outputPathId = PotionSubtypeInterpreter.INSTANCE.getStringName(potionOutput);
				String outputModId = outputResourceLocation.method_12836();
				String uidPath = ResourceLocationUtil.sanitizePath(inputPathId + ".to." + outputPathId);
				IJeiBrewingRecipe recipe = recipeFactory.createBrewingRecipe(
					List.of(potionReagent),
					potionInputCopy,
					potionOutput,
					class_2960.method_60655(outputModId, uidPath)
				);

				IJeiBrewingRecipe existingRecipe = recipes.stream()
					.filter(recipe::equals)
					.findFirst()
					.orElse(null);
				if (existingRecipe == null) {
					recipes.add(recipe);
					newPotions.add(potionOutput);
				} else {
					// This is a recipe with the same uid and output as an existing recipe,
					// but it has a different reagent.
					// Create a recipe that combines the two.
					IngredientSet<class_1799> reagents = new IngredientSet<>(itemStackHelper, UidContext.Recipe);
					reagents.addAll(existingRecipe.getIngredients());
					reagents.add(potionReagent);
					if (reagents.size() != existingRecipe.getIngredients().size()) {
						IJeiBrewingRecipe replacementRecipe = recipeFactory.createBrewingRecipe(
							List.copyOf(reagents),
							existingRecipe.getPotionInputs(),
							existingRecipe.getPotionOutput(),
							existingRecipe.getUid()
						);
						recipes.remove(existingRecipe);
						recipes.add(replacementRecipe);
					}
				}
			}
		}
		return newPotions;
	}

	private static class_1799 getOutput(class_1845 potionBrewing, class_1799 potion, class_1799 itemStack) {
		try {
			class_1799 result = potionBrewing.method_8078(itemStack, potion);
			if (result != itemStack) {
				return result;
			}
		} catch (RuntimeException e) {
			String potionInfo = ErrorUtil.getItemStackInfo(potion);
			String itemStackInfo = ErrorUtil.getItemStackInfo(itemStack);
			LOGGER.error(
				"A modded potion mix crashed: \nPotion: {}\nItemStack: {}",
				potionInfo,
				itemStackInfo,
				e
			);
		}
		return class_1799.field_8037;
	}
}
